package org.ten60.wink.persist.endpoint;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

import java.io.*;
import java.net.URI;
import java.text.DateFormat;
import java.util.Date;

public class FilePersistAccessor extends StandardAccessorImpl
{
	public FilePersistAccessor()
	{	this.declareThreadSafe();		
	}

	@Override
	public void onSink(INKFRequestContext aContext) throws Exception
	{	String base=getFileURI(aContext);
		String changes=base+".changes";
		File c=new File(URI.create(changes));
		int version=1;
		if(c.exists())
		{	String lastLine=getLastLine(c);
			String[] parts=lastLine.split(",");
			version=Integer.parseInt(parts[0])+1;
		}
		else
		{	File dir=c.getParentFile();
			if(!dir.exists())
			{	dir.mkdirs();				
			}
			c.createNewFile();
		}
		
		BufferedWriter bw=new BufferedWriter(new FileWriter(c,true));
		try
		{	String user=(String)aContext.source("session:userid",String.class);
			bw.append(version+","+user+","+System.currentTimeMillis());
			bw.newLine();
			aContext.sink(base+"."+version, aContext.getThisRequest().getPrimary());
			aContext.sink(base+".latest",aContext.getThisRequest().getPrimary());
		}
		finally
		{	bw.close();			
		}
	}

	public void onSource(INKFRequestContext aContext) throws Exception
	{	File f;
		String theURI = getFileURI(aContext);
		if (theURI.charAt(theURI.length()-1) == '/')
		{	f=new File(URI.create(getFileURI(aContext)));
		}
		else
		{	f=new File(URI.create(theURI + ".latest"));
		}
		String wiki=aContext.getThisRequest().getArgumentValue("wiki");
		if(f.isDirectory())
		{	INKFResponse resp=aContext.createResponseFrom(getDirectoryListing(f,wiki));
			resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
		}
		else
		{	String fileURI=getFileURI(aContext);
			if(fileURI.endsWith(".changes"))
			{	HDSBuilder b=new HDSBuilder();
				b.pushNode("changes");
					b.pushNode("wiki",wiki);
					b.popNode();
				File c=new File(URI.create(fileURI));
				if(c.exists())
				{	BufferedReader br=new BufferedReader(new FileReader(c));
					String line=null;
					while((line=br.readLine())!=null)
					{	String[] sa=line.split(",");
						b.pushNode("change");
							b.pushNode("rev",sa[0]);
							b.popNode();
							b.pushNode("user",sa[1]);
							b.popNode();
							Date d=new Date(Long.parseLong(sa[2]));
							DateFormat df=DateFormat.getDateTimeInstance();
							b.pushNode("date", df.format(d));
							b.popNode();
						b.popNode();
					}
				}
				b.popNode();
				INKFResponse resp=aContext.createResponseFrom(b.getRoot());
				resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
			}
			else
			{	if(!fileURI.matches(".*?\\.(\\d*|latest)"))
				{	fileURI=fileURI+".latest";
				}
				try
				{	INKFResponseReadOnly resp=aContext.sourceForResponse(fileURI);
					aContext.createResponseFrom(resp);
				}
				catch(NKFException e)
				{	if(e.getDeepestId().equals("java.io.FileNotFoundException"))
					{	throw new NKFException("No Wiki Resource Yet","File not found");
					}
					else throw e;
				}
			}
		}
	}
	
	private String getFileURI(INKFRequestContext aContext) throws Exception
	{	String wiki=aContext.getThisRequest().getArgumentValue("wiki");
		String dir=(String)aContext.source("active:HDSXpath+operand@fpds:/etc/WinkConfig.xml+operator@/wink/persistFile/directory/text()", String.class);
		return dir+wiki;
	}
	
	private IHDSNode getDirectoryListing(File f, String wiki)
	{	HDSBuilder b=new HDSBuilder();
		b.pushNode("directory");
			if(wiki.length()>0)
			{	b.pushNode("wiki",wiki);
				b.popNode();
			}
		for(File i: f.listFiles())
		{	if(i.isDirectory())
			{	b.pushNode("dir", i.getName());
				b.popNode();
			}
			else if(i.getName().endsWith(".latest"))
			{	b.pushNode("file", i.getName().substring(0, i.getName().length()-7));
				b.popNode();
			}
		}
		b.popNode();
		return b.getRoot();
	}
	
	/**
	 *  A simple way to get the last line
	 * @param aFile
	 * @return String
	 */
	 private String getLastLine(File aFile) throws Exception
	 {	BufferedReader aReader=new BufferedReader(new FileReader(aFile));
		 String line=null, tmp ;
	 	try
		{	while ((tmp = aReader.readLine()) != null)
			{	line = tmp ;
			}
		}
	 	catch (IOException e)
		{	//do nothing
	    }
		return line ;
	}
}
